#!/bin/sh
# bbwarn.sh
# Sean MacGuire - The MacLawran Group Inc.
# Apr 17, 1998
#
# This program is Copyright (c) 1998
# Robert-Andre Croteau, The MacLawran Group Inc.
# All Rights Reserved
#

# BIG BROTHER MONITORING SCRIPT
#
# For each entry in the $BBHOME/etc/bbwarn.cfg
#
# check if a resource hit the red condition
#

if [ "$BBWARN" != "TRUE" ]
then
	exit 0				# No warning to be issued
fi

if test "$#" != "1"
then
    echo "bbwarn.sh: wrong argument count - $#"
    exit 1
else
    # Pick the machine.service combination from the msg
    ARGS="$1"
    set $1
    if test "$#" -lt "3"
    then
	echo "bbwarn.sh: wrong argument count within message"
	exit 2
    fi
    MACHSRV=$1   # i.e. hp1.cpu
fi
if test ! "$BBTMP"                      # GET DEFINITIONS IF NEEDED
then
	# echo "*** LOADING BBDEF ***"
        . $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

if [ ! -f $BBHOME/etc/bbwarn.cfg ]
then
	exit 1
else
	set -f		# disable globber
	ignline=`grep "^ignforall:" $BBHOME/etc/bbwarn.cfg`
	if [ "$?" = 0 ]
	then
		set $ignline
		shift
		ignhosts=$*
		echo $MACHSRV | $EGREP -x "$ignhosts" | $GREP -v $EGREP
		if [ "$?" = 0 ]
		then
			exit 0
		fi
	fi
	set +f		# enable globber (just in case)
	cd $BBHOME/www/logs/
	# date is 0-6 (sunday-saturday)
	nowday=`$DATE +"%w"`
	# time is 0000-2359
	nowtime=`$DATE +"%H%M"`
	if [ "$?" = 0 ]
	then
		$CAT $BBHOME/etc/bbwarn.cfg | $GREP -v "^#" | $GREP -v "ignforall:" | \
		while read line
		do
			if [ -z "$line" ]
			then
				continue
			fi
			OLDIFS=$IFS
			IFS=','
			set -f 		# disable globber we may have a * in the line
			set $line
			IFS=$OLDIFS
			hosts="$1"
			if [ "$2" != "" ]
			then
				exhosts=""
				for exhost in $2
				do
					if [ "$exhosts" = "" ]
					then
						exhosts="$exhost.*"
					else
						exhosts="$exhosts|$exhost.*"
					fi
				done
			else
				exhosts="999999999"
			fi
			svcs="$3"
			if [ "$4" != "" ]
			then
				exsvcs=""
				for exsvc in $4
				do
					if [ "$exsvcs" = "" ]
					then
						exsvcs=".*.$exsvc"
					else
						exsvcs="$exsvcs|.*.$exsvc"
					fi
				done
			else
				exsvcs="999999999"
			fi
			set +f 		# reenable globber
			days="$5"
			times="$6"
			PAGER="$7"
			notifyuser=FALSE
			echo $MACHSRV | $EGREP -vx "$exhosts" | $EGREP -vx "$exsvcs" | $GREP -v $EGREP | { \
			while read redfile
			do
				OLDIFS=$IFS
				IFS='.'
				set $redfile
				IFS=$OLDIFS
				if test "$FQDN" = "TRUE"
				then
					redhost=`echo $1 | $SED "s/,/\./g"`
				else
					redhost=$1
				fi
				redsvc=$2
				notify=FALSE
				if [ "$hosts" = '*' ]
				then
					$TRUE
				else
					echo $hosts | $GREP $redhost >/dev/null 2>&1
				fi
				if [ "$?" = 0 ]
				then
					if [ "$svcs" = '*' ]
					then
						$TRUE
					else
						echo $svcs | $GREP $redsvc >/dev/null 2>&1
					fi
					if [ "$?" = 0 ]
					then
						set -f		# disable globber
						for day in $days
						do
							validday=FALSE
							OLDIFS=$IFS
							IFS='-'
							set $day
							IFS=$OLDIFS
							if [ "$#" -eq 2 ]
							then
								if [ $nowday -ge $1 -a $nowday -le $2 ]
								then
									validday=TRUE
								fi
							elif [ "$#" -eq 1 ]
							then
								if [ '*' = $day ]
								then
									validday=TRUE
								elif [ $nowday -eq $day ]
								then
									validday=TRUE
								fi
							fi
							if [ "$validday" = "TRUE" ]
							then
								for time in $times
								do
									validtime=FALSE
									OLDIFS=$IFS
									IFS='-'
									set $time
									IFS=$OLDIFS
									if [ "$#" -eq 2 ]
									then
										if [ $nowtime -ge $1 -a $nowtime -le $2 ]
										then
											validtime=TRUE
										fi
									elif [ "$#" -eq 1 ]
									then
										if [ '*' = $time ]
										then
											validtime=TRUE
										elif [ $nowtime -eq $time ]
										then
											validtime=TRUE
										fi
									fi
									if [ "$validtime" = "TRUE" ]
									then
										notify=TRUE
										break
									fi
								done # on time
							fi
						done # on day 
						set +f		# reenable globber
					fi # on service
				fi # on host
				# Stop on first broken service (no need to continue)
				if [ "$notify" = "TRUE" ]
				then
					notifyuser=TRUE
					break
				fi
			done # on host list with red
			if [ "$notifyuser" = "TRUE" ]
			then
				if [ ! -z "$PAGER" ]
				then
					export PAGER
					# PAGER variable contains recipients arg $7

					# Get IP address of host
					$GREP $redhost $BBHOSTS | \
					while read hostline
					do
						set $hostline
						if [ "$#" -ge "2" ]   # Make sure it's a valid line (well at least 2 args)
						then
							MACHIP=`echo $1 | $TR '.' ' '| $AWK '{printf("%03d%03d%03d%03d",$1,$2,$3,$4)}'`
							if test "$FQDN" = "TRUE"
							then
								set $2
							else
								OLDIFS=$IFS
								IFS='.'
								set $2
								IFS=$OLDIFS
							fi
							if [ "$redhost" = "$1" ]
							then
								SVCERR=""
								for svcerr in $SVCERRLIST  # Get proper code
								do
									OLDIFS=$IFS
									IFS=':'
									set $svcerr
									IFS=$OLDIFS
									if [ "$1" = "$redsvc" ]
									then
										SVCERR=$2
										break
									fi
								done
								# Now do your stuff
								if test "$FQDN" = "TRUE"
								then
									set $ARGS
									ARG1=$1
									shift
									ARG1=`echo $ARG1 | $SED "s/,/\./g"`
									ARGS="$ARG1 $*"
								fi
								$BBHOME/bin/bb-page.sh "${SVCERR}${MACHIP} $ARGS"
								break
							fi
						fi
					done
				fi
			fi
			}
		done # on config file
	fi
fi

exit 0
